!***********************************************************************
!                        Version 1:  03/2007, PFN                      *
!                                                                      *
!   Quadrature Data Module                                             *
!                                                                      *
!   Level-Symmetric quadrature data used in multidimensional geometry  *
!                                                                      *
!   Allowed values of "n" are:  2, 4, 6, 8, 10, 12, 14, 16, 18, 20     *
!                                                                      *
!   Directions per Octant:                                             *
!                                                                      *
!                             N   N(N+2)/8                             *
!                             2       1                                *
!                             4       3                                *
!                             6       6                                *
!                             8      10                                *
!                            10      15                                *
!                            12      21                                *
!                            14      28                                *
!                            16      36                                *
!                            18      45                                *
!                            20      55                                *
!                                                                      *
!***********************************************************************

   module QuadratureData_mod 

   use kind_mod
   use constant_mod

   private 

!  Public Interfaces 

!  Level/Fully Symmetric Quadrature
   integer, public    :: iang(20)
   integer, public    :: icoff(20)
   integer, public    :: imu(220)
   integer, public    :: ieta(220)
   integer, public    :: ixi(220)
   integer, public    :: iwgt(220)

   real(adqt), public :: dircos(58)
   real(adqt), public :: weight(58)

!  Product Quadrature
   integer, public    :: first(22)
   integer, public    :: last(22)

   real(adqt), public :: cosTheta(171)
   real(adqt), public :: weightTheta(171)
   real(adqt), public :: cosPhiXYZ(253)
   real(adqt), public :: weightPhiXYZ(253)

!  Constants

!  All quadrature sets are Level-Symmetric except for S20 which
!  is Fully-Symmetric (only sets with all positive points weights
!  are included).  The arrays DIRCOS and WEIGHT have been padded
!  with zeros to align the quadrature sets.

      data dircos/                         &
!  S2             
                  0.577350269189625_adqt,  &
!  S4
                  0.868890300722201_adqt,  &
                  0.350021174581540_adqt,  &
!  S6
                  0.926180935517489_adqt,  &
                  0.681507726536546_adqt,  &
                  0.266635401516704_adqt,  &
!  S8
                  0.951189731211341_adqt,  &
                  0.786795792469443_adqt,  &
                  0.577350269189625_adqt,  &
                  0.218217890235992_adqt,  &
!  S10 
                  0.963490981110468_adqt,  &
                  0.839759962236684_adqt,  &
                  0.694318887594384_adqt,  &
                  0.508881755582618_adqt,  &
                  0.189321326478010_adqt,  &
!  S12
                  0.971637719251358_adqt,  &
                  0.872270543025721_adqt,  &
                  0.760021014833664_adqt,  &
                  0.628019096642130_adqt,  &
                  0.459547634642594_adqt,  &
                  0.167212652822713_adqt,  &
!  S14
                  0.976627152925770_adqt,  &
                  0.893691098874356_adqt,  &
                  0.802226255231412_adqt,  &
                  0.698892086775901_adqt,  &
                  0.577350269189625_adqt,  &
                  0.422156982304796_adqt,  &
                  0.151985861461031_adqt,  &
!  S16
                  0.980500879011739_adqt,  &
                  0.909285500943725_adqt,  &
                  0.831996556910044_adqt,  &
                  0.746750573614681_adqt,  &
                  0.650426450628771_adqt,  &
                  0.537096561300879_adqt,  &
                  0.392289261444811_adqt,  &
                  0.138956875067780_adqt,  &
!  S18
                  0.983127661236087_adqt,  &
                  0.920768021061018_adqt,  &
                  0.853866206691488_adqt,  &
                  0.781256199495913_adqt,  &
                  0.701166884252161_adqt,  &
                  0.610662549934881_adqt,  &
                  0.504165151725164_adqt,  &
                  0.368043816053393_adqt,  &
                  0.129344504545924_adqt,  &
                  0.0_adqt,                &
!  S20 (Fully Symmetric)
                  0.985347485558646_adqt,  &
                  0.929863938955324_adqt,  &
                  0.870852583760463_adqt,  &
                  0.807540401661143_adqt,  &
                  0.738822561910371_adqt,  &
                  0.663020403653288_adqt,  &
                  0.577350269189625_adqt,  &
                  0.476519266143665_adqt,  &
                  0.347574292315847_adqt,  &
                  0.120603343036693_adqt,  &
                  0.0_adqt,                &
                  0.0_adqt                 /


      data weight /                        &
!  S2           
                   1.000000000000000_adqt, &
!  S4
                   0.333333333333333_adqt, &
                   0.0_adqt,               &
!  S6
                   0.176126130863383_adqt, &
                   0.157207202469949_adqt, &
                   0.0_adqt,               &
!  S8
                   0.120987654320987_adqt, &
                   0.090740740740740_adqt, &
                   0.092592592592592_adqt, &
                   0.0_adqt,               &
!  S10
                   0.089303147984356_adqt, &
                   0.072529151712365_adqt, &
                   0.045043767436408_adqt, &
                   0.053928114487836_adqt, &
                   0.0_adqt,               &
!  S12
                   0.070762589970091_adqt, &
                   0.055881101564888_adqt, &
                   0.037337673758828_adqt, &
                   0.050281901060057_adqt, &
                   0.025851291655750_adqt, &
                   0.0_adqt,               &
!  S14
                   0.057997040896996_adqt, &
                   0.048900797636810_adqt, &
                   0.022793534241187_adqt, &
                   0.039413200595007_adqt, &
                   0.038099086144012_adqt, &
                   0.025839407641890_adqt, &
                   0.008269579972622_adqt, &
!  S16
                   0.048987239158038_adqt, &
                   0.041329597869844_adqt, &
                   0.020303200739365_adqt, &
                   0.026550075781349_adqt, &
                   0.037907440795600_adqt, &
                   0.013529504778675_adqt, &
                   0.032636937202685_adqt, &
                   0.010376957838539_adqt, &
!  S18
                   0.042264644884382_adqt, &
                   0.037612747382728_adqt, &
                   0.012269135163740_adqt, &
                   0.032418835255881_adqt, &
                   0.006644386146190_adqt, &
                   0.031209383843655_adqt, &
                   0.016012725269194_adqt, &
                   0.020048459530857_adqt, &
                   0.000111409402059_adqt, &
                   0.016379703852242_adqt, &
!  S20 (Fully Symmetric)
                   0.037021049060448_adqt, &
                   0.033284216537631_adqt, &
                   0.012942568580374_adqt, &
                   0.022287358265511_adqt, &
                   0.014054150411563_adqt, &
                   0.028265562625927_adqt, &
                   0.009085313444449_adqt, &
                   0.031556484065954_adqt, &
                   0.000982552577261_adqt, &
                   0.003031445199340_adqt, &
                   0.003161892186100_adqt, &
                   0.033866270661553_adqt  /


      data iwgt/1,   &
!  S4
                1,1, &
                1,   &
!  S6
                1,2,1, &
                2,2,   &
                1,     &
!  S8 
                1,2,2,1, &
                2,3,2,   &
                2,2,     &
                1,       &
!  S10
                1,2,3,2,1, &
                2,4,4,2,   &
                3,4,3,     &
                2,2,       &
                1,         &
!  S12
                1,2,3,3,2,1, &
                2,4,5,4,2,   &
                3,5,5,3,     &
                3,4,3,       &
                2,2,         &
                1,           &
!  S14
                1,2,3,4,3,2,1, &
                2,5,6,6,5,2,   &
                3,6,7,6,3,     &
                4,6,6,4,       &
                3,5,3,         &
                2,2,           &
                1,             &
!  S16
                1,2,3,4,4,3,2,1, &
                2,5,6,7,6,5,2,   &
                3,6,8,8,6,3,     &
                4,7,8,7,4,       &
                4,6,6,4,         &
                3,5,3,           &
                2,2,             &
                1,               &
!  S18
                1, 2, 3, 4, 5, 4, 3, 2, 1, &
                2, 6, 7, 8, 8, 7, 6, 2,    &
                3, 7, 9,10, 9, 7, 3,       &
                4, 8,10,10, 8, 4,          &
                5, 8, 9, 8, 5,             &
                4, 7, 7, 4,                &
                3, 6, 3,                   &
                2, 2,                      &
                1,                         &                    
!  S20
                1, 2, 3, 4, 5, 5, 4, 3, 2, 1, &
                2, 6, 7, 8, 9, 8, 7, 6, 2,    &
                3, 7,10,11,11,10, 7, 3,       &
                4, 8,11,12,11, 8, 4,          &
                5, 9,11,11, 9, 5,             &
                5, 8,10, 8, 5,                &
                4, 7, 7, 4,                   &
                3, 6, 3,                      &
                2, 2,                         & 
                1/

      data imu/1,   &
!  S4
               1,2, &
               2,   &
!  S6
               1,2,3, &
               2,3,   &
               3,     &
!  S8
               1,2,3,4, &
               2,3,4,   &
               3,4,     &
               4,       &
!  S10
               1,2,3,4,5, &
               2,3,4,5,   &
               3,4,5,     &
               4,5,       &
               5,         &
!  S12
               1,2,3,4,5,6, &
               2,3,4,5,6,   &
               3,4,5,6,     &
               4,5,6,       &
               5,6,         &
               6,           &
!  S14
               1,2,3,4,5,6,7, &
               2,3,4,5,6,7,   &
               3,4,5,6,7,     &
               4,5,6,7,       &
               5,6,7,         &
               6,7,           &
               7,             &
!  S16
               1,2,3,4,5,6,7,8, &
               2,3,4,5,6,7,8,   &
               3,4,5,6,7,8,     &
               4,5,6,7,8,       &
               5,6,7,8,         &
               6,7,8,           &
               7,8,             &
               8,               &
!  S18
               1,2,3,4,5,6,7,8,9, &
               2,3,4,5,6,7,8,9,   &
               3,4,5,6,7,8,9,     &
               4,5,6,7,8,9,       &
               5,6,7,8,9,         &
               6,7,8,9,           &
               7,8,9,             &
               8,9,               &
               9,                 &
!  S20
               1,2,3,4,5,6,7,8,9,10, &
               2,3,4,5,6,7,8,9,10,   &
               3,4,5,6,7,8,9,10,     &
               4,5,6,7,8,9,10,       &
               5,6,7,8,9,10,         &
               6,7,8,9,10,           & 
               7,8,9,10,             &
               8,9,10,               &
               9,10,                 &
               10/

      data ieta/1,   &
!  S4
                2,2, &
                1,   &
!  S6
                3,3,3, &
                2,2,   &
                1,     &
!  S8
                4,4,4,4, &
                3,3,3,   &
                2,2,     &
                1,       &
!  S10
                5,5,5,5,5, &
                4,4,4,4,   &
                3,3,3,     &
                2,2,       &
                1,         &
!  S12
                6,6,6,6,6,6, &
                5,5,5,5,5,   &
                4,4,4,4,     &
                3,3,3,       &
                2,2,         &
                1,           &
!  S14
                7,7,7,7,7,7,7, &
                6,6,6,6,6,6,   &
                5,5,5,5,5,     &
                4,4,4,4,       &
                3,3,3,         &
                2,2,           &
                1,             &
!  S16
                8,8,8,8,8,8,8,8, &
                7,7,7,7,7,7,7,   &
                6,6,6,6,6,6,     &
                5,5,5,5,5,       &
                4,4,4,4,         &
                3,3,3,           &
                2,2,             &
                1,               &
!  S18
                9,9,9,9,9,9,9,9,9, &
                8,8,8,8,8,8,8,8,   &
                7,7,7,7,7,7,7,     &
                6,6,6,6,6,6,       &
                5,5,5,5,5,         &
                4,4,4,4,           &
                3,3,3,             &
                2,2,               &
                1,                 &
!  S20
                10,10,10,10,10,10,10,10,10,10, &
                9,9,9,9,9,9,9,9,9,             &
                8,8,8,8,8,8,8,8,               &
                7,7,7,7,7,7,7,                 &
                6,6,6,6,6,6,                   &
                5,5,5,5,5,                     &
                4,4,4,4,                       &
                3,3,3,                         &
                2,2,                           &
                1/

      data ixi/1,   &
!  S4
               2,1, &
               2,   &
!  S6
               3,2,1, &
               3,2,   &
               3,     &
!  S8
               4,3,2,1, &
               4,3,2,   &
               4,3,     &
               4,       &
!  S10
               5,4,3,2,1, &
               5,4,3,2,   &
               5,4,3,     &
               5,4,       &
               5,         &
!  S12
               6,5,4,3,2,1, &
               6,5,4,3,2,   &
               6,5,4,3,     &
               6,5,4,       &
               6,5,         &
               6,           &
!  S14
               7,6,5,4,3,2,1, &
               7,6,5,4,3,2,   &
               7,6,5,4,3,     &
               7,6,5,4,       &
               7,6,5,         &
               7,6,           &
               7,             &
!  S16
               8,7,6,5,4,3,2,1, &
               8,7,6,5,4,3,2,   &
               8,7,6,5,4,3,     &
               8,7,6,5,4,       &
               8,7,6,5,         &
               8,7,6,           &
               8,7,             &
               8,               &
!  S18
               9,8,7,6,5,4,3,2,1, &
               9,8,7,6,5,4,3,2,   &
               9,8,7,6,5,4,3,     &
               9,8,7,6,5,4,       &
               9,8,7,6,5,         & 
               9,8,7,6,           &
               9,8,7,             &
               9,8,               &
               9,                 &
!  S20
               10,9,8,7,6,5,4,3,2,1, &
               10,9,8,7,6,5,4,3,2,   &
               10,9,8,7,6,5,4,3,     &
               10,9,8,7,6,5,4,       &
               10,9,8,7,6,5,         &
               10,9,8,7,6,           &
               10,9,8,7,             &
               10,9,8,               &
               10,9,                 &
               10/       

!  Offsets into IMU, IETA, IXI and IWGT arrays

   data iang /0,1,0,2,0,5,0,11,0,21,  &
              0,36,0,57,0,85,0,121,0,166/

!  Offsets into the DIRCOS and WEIGHT arrays

   data icoff /0,0,0,1,0,3,0,6,0,10,  &
               0,15,0,21,0,28,0,36,0,46/

!  Data for Product Quadrature. "First" and "Last" are
!  ranges for the tabulated cosines and weights.

   data first /1,2,4,7,11,16,22,29,37,46,56,67,  &
               79,92,106,121,137,154,172,191,211,232/
   data last  /1,3,6,10,15,21,28,36,45,55,66,78, &
               91,105,120,136,153,171,190,210,231,253/

!  Quadratures for Polar Integration in R-Z Cylindrical and X-Y-Z Geometry

   data cosTheta /                       &
!  N=1
                 0.6189908924467_adqt,   &
!  N=2
                 0.8634990514471_adqt,   &
                 0.2707467653586_adqt,   &
!  N=3
                 0.9420141575125_adqt,   &
                 0.6014839209434_adqt,   &
                 0.1436154828008_adqt,   &
!  N=4
                 0.9692550874731_adqt,   &
                 0.7603841957105_adqt,   &
                 0.3781223125864_adqt,   &
                 0.07288031758736_adqt,  &
!  N=5
                 0.9907228027705_adqt,   &
                 0.9066754949760_adqt,   &
                 0.6763407250056_adqt,   &
                 0.3455719708920_adqt,   &
                 0.07412304396110_adqt,  &
!  N=6
                 0.9934260252776_adqt,   &
                 0.9365569210587_adqt,   &
                 0.7781929016189_adqt,   &
                 0.5233269543727_adqt,   &
                 0.2469213715865_adqt,   &
                 0.05065384196986_adqt,  &
!  N=7
                 0.9957315827450_adqt,   &
                 0.9579357169756_adqt,   &
                 0.8474708404724_adqt,   &
                 0.6540152494700_adqt,   &
                 0.4119940170234_adqt,   &
                 0.1854112547976_adqt,   &
                 0.03705166699011_adqt,  &
!  N=8
                 0.9970063368207_adqt,   &
                 0.9699517300226_adqt,   &
                 0.8879503955450_adqt,   &
                 0.7361944193605_adqt,   &
                 0.5304474814371_adqt,   &
                 0.3126552110074_adqt,   &
                 0.1323869707831_adqt,   &
                 0.02526885705460_adqt,  &

!  N=9
                 0.9985935590648_adqt,   &
                 0.9848612009152_adqt,   &
                 0.9384448738513_adqt,   &
                 0.8409695379867_adqt,   &
                 0.6884671704442_adqt,   &
                 0.4968907718281_adqt,   &
                 0.2982703761263_adqt,   &
                 0.1304306562034_adqt,   &
                 0.02572642057725_adqt,  &
!  N=10
                 0.9988431905941_adqt,   &
                 0.9878187060973_adqt,   &
                 0.9512982489952_adqt,   &
                 0.8748210152191_adqt,   &
                 0.7528387780840_adqt,   &
                 0.5927047772245_adqt,   &
                 0.4137049559308_adqt,   &
                 0.2420294039117_adqt,   &
                 0.1040133416870_adqt,   &
                 0.02032527456069_adqt,  &
!  N=11
                 0.9991302677320_adqt,   &
                 0.9907904599945_adqt,   &
                 0.9628342127515_adqt,   &
                 0.9031675682025_adqt,   &
                 0.8053034593427_adqt,   &
                 0.6716836707816_adqt,   &
                 0.5139106257595_adqt,   &
                 0.3501888569958_adqt,   &
                 0.2011107021879_adqt,   &
                 0.08533860285514_adqt,  &
                 0.01656084164276_adqt,  &
!  N=12
                 0.9993182903074_adqt,   &
                 0.9927399491150_adqt,   &
                 0.9704415432003_adqt,   &
                 0.9220724109667_adqt,   &
                 0.8409912076571_adqt,   &
                 0.7271112553886_adqt,   &
                 0.5876609071491_adqt,   &
                 0.4358871441498_adqt,   &
                 0.2882445618572_adqt,   &
                 0.1609145368599_adqt,   &
                 0.06661300974079_adqt,  &
                 0.01271017917531_adqt,  &

!  N=13
                 0.9996056789222_adqt,   &
                 0.9956783395241_adqt,   &
                 0.9817170248781_adqt,   &
                 0.9496614980081_adqt,   &
                 0.8924880178733_adqt,   &
                 0.8066758436297_adqt,   &
                 0.6937364298650_adqt,   &
                 0.5604409274254_adqt,   &
                 0.4178107622661_adqt,   &
                 0.2792375210342_adqt,   &
                 0.1582700765155_adqt,   &
                 0.06654296956713_adqt,  &
                 0.01288229858096_adqt,  &
!  N=14
                 0.9996558737820_adqt,   &
                 0.9962693383644_adqt,   &
                 0.9844006381495_adqt,   &
                 0.9574371978087_adqt,   &
                 0.9095151164539_adqt,   &
                 0.8372368352245_adqt,   &
                 0.7407906029852_adqt,   &
                 0.6242752767306_adqt,   &
                 0.4952311643392_adqt,   &
                 0.3635415417531_adqt,   &
                 0.2399856722056_adqt,   &
                 0.1347776768636_adqt,   &
                 0.05639551887688_adqt,  &
                 0.01086656296514_adqt,  &
!  N=15 
                 0.9997217860289_adqt,   &
                 0.9969770851516_adqt,   &
                 0.9873133178911_adqt,   &
                 0.9652017991798_adqt,   &
                 0.9255042701452_adqt,   &
                 0.8648133493431_adqt,   &
                 0.7823872347417_adqt,   &
                 0.6805340155142_adqt,   &
                 0.5644276467831_adqt,   &
                 0.4414412603907_adqt,   &
                 0.3201611900935_adqt,   &
                 0.2092924891631_adqt,   &
                 0.1166739455867_adqt,   &
                 0.04857478250334_adqt,  &
                 0.009333807675231_adqt, &

!  N=16
                 0.9997688033812_adqt,   &
                 0.9974817188842_adqt,   &
                 0.9893904569635_adqt,   &
                 0.9707482915922_adqt,   &
                 0.9369723698241_adqt,   &
                 0.8847379407654_adqt,   &
                 0.8127896715603_adqt,   &
                 0.7223510099119_adqt,   &
                 0.6171016248194_adqt,   &
                 0.5027643330792_adqt,   &
                 0.3863961603479_adqt,   &
                 0.2755133143512_adqt,   &
                 0.1771953459552_adqt,   &
                 0.09731207205116_adqt,  &
                 0.04001376430336_adqt,  &
                 0.007627772606466_adqt, &
!  N=17
                 0.9998496261600_adqt,   &
                 0.9983377240080_adqt,   &
                 0.9928507662419_adqt,   &
                 0.9798024424910_adqt,   &
                 0.9553151902739_adqt,   &
                 0.9160087673050_adqt,   &
                 0.8597217960590_adqt,   &
                 0.7860255025427_adqt,   &
                 0.6964538247577_adqt,   &
                 0.5944358823670_adqt,   &
                 0.4849651757731_adqt,   &
                 0.3740752050608_adqt,   &
                 0.2682147415396_adqt,   &
                 0.1736297383459_adqt,   &
                 0.09591061666473_adqt,  &
                 0.03889663020451_adqt,  &
                 0.007709154866395_adqt, &
!  N=18 
                 0.9998643186575_adqt,   &
                 0.9985095657004_adqt,   &
                 0.9936394403828_adqt,   &
                 0.9821650338648_adqt,   &
                 0.9607790068479_adqt,   &
                 0.9265617655799_adqt,   &
                 0.8775185305293_adqt,   &
                 0.8129709750266_adqt,   &
                 0.7337588102852_adqt,   &
                 0.6422382246552_adqt,   &
                 0.5420919860080_adqt,   &
                 0.4379889494132_adqt,   &
                 0.3351477918895_adqt,   &
                 0.2388696773826_adqt,   &
                 0.1541052327805_adqt,   &
                 0.08511044202699_adqt,  &
                 0.03522248344992_adqt,  &
                 0.006746971030014_adqt  /

   data weightTheta /                     &
!  N=1
                 1.000000000000_adqt,     &
!  N=2
                 0.3867606081357_adqt,    &
                 0.6132393918643_adqt,    &
!  N=3
                 0.1734817853566_adqt,    &
                 0.4758504223783_adqt,    &
                 0.3506677922651_adqt,    &
!  N=4
                 0.09483958290718_adqt,   &
                 0.3239509561641_adqt,    &
                 0.3911333839368_adqt,    &
                 0.1900760769918_adqt,    &
!  N=5
                 0.03066865270574_adqt,   &
                 0.1520873796776_adqt,    &
                 0.3007989398347_adqt,    &
                 0.3315500881013_adqt,    &
                 0.1848949396807_adqt,    &
!  N=6
                 0.02136152921019_adqt,   &
                 0.1021655746368_adqt,    &
                 0.2139690494731_adqt,    &
                 0.2824180551767_adqt,    &
                 0.2523904145543_adqt,    &
                 0.1276953769490_adqt,    &
!  N=7
                 0.01392973976943_adqt,   &
                 0.06892898464294_adqt,   &
                 0.1540022963383_adqt,    &
                 0.2268755020184_adqt,    &
                 0.2459222632522_adqt,    &
                 0.1963577927660_adqt,    &
                 0.09398342121276_adqt,   &
!  N=8
                 0.009809563177794_adqt,  &
                 0.04993755387300_adqt,   &
                 0.1167900832303_adqt,    &
                 0.1840077166009_adqt,    &
                 0.2201579963795_adqt,    &
                 0.2067710539408_adqt,    &
                 0.1476215292233_adqt,    &
                 0.06490450357423_adqt,   &
!  N=9
                 0.004687271581718_adqt,  &
                 0.02638070556035_adqt,   &
                 0.06974359023450_adqt,   &
                 0.1259654703863_adqt,    &
                 0.1763082378214_adqt,    &
                 0.2013205178931_adqt,    &
                 0.1894159794100_adqt,    &
                 0.1407412491186_adqt,    &
                 0.06543697799409_adqt,   &

!  N=10 
                 0.003831690452384_adqt,  &
                 0.02096950939459_adqt,   &
                 0.05461898814248_adqt,   &
                 0.09932946005030_adqt,   &
                 0.1433909126899_adqt,    &
                 0.1735729930089_adqt,    &
                 0.1799419832720_adqt,    &
                 0.1589187774698_adqt,    &
                 0.1136148860623_adqt,    &
                 0.05181079945732_adqt,   &
!  N=11
                 0.002884161734127_adqt,  &
                 0.01592166778168_adqt,   &
                 0.04210429510321_adqt,   &
                 0.07836749831056_adqt,   &
                 0.1169567716752_adqt,    &
                 0.1483035686839_adqt,    &
                 0.1641342683895_adqt,    &
                 0.1598016376376_adqt,    &
                 0.1351922425517_adqt,    &
                 0.09405053253703_adqt,   &
                 0.04228335559564_adqt,   &
!  N=12
                 0.002263464406185_adqt,  &
                 0.01260438060930_adqt,   &
                 0.03379182957262_adqt,   &
                 0.06410234109849_adqt,   &
                 0.09808702825277_adqt,   &
                 0.1284451713994_adqt,    &
                 0.1481987487236_adqt,    &
                 0.1525460148381_adqt,    &
                 0.1399867016900_adqt,    &
                 0.1125198356935_adqt,    &
                 0.07486149506603_adqt,   &
                 0.03259298864993_adqt,   &
!  N=13
                 0.001317932282413_adqt,  &
                 0.007651720102310_adqt,  &
                 0.02166000129594_adqt,   &
                 0.04366484358444_adqt,   &
                 0.07129522897898_adqt,   &
                 0.1000807915314_adqt,    &
                 0.1246517659564_adqt,    &
                 0.1400696627407_adqt,    &
                 0.1429204365193_adqt,    &
                 0.1319338252731_adqt,    &
                 0.1080243650166_adqt,    &
                 0.07381055437147_adqt,   &
                 0.03291887234697_adqt,   &

!  N=14
                 0.001147057513783_adqt,  &
                 0.006560585266642_adqt,  &
                 0.01831225051549_adqt,   &
                 0.03662153822498_adqt,   &
                 0.05980800643841_adqt,   &
                 0.08472257845895_adqt,   &
                 0.1074785132224_adqt,    &
                 0.1242764203624_adqt,    &
                 0.1321396462151_adqt,    &
                 0.1294179363033_adqt,    &
                 0.1159714807527_adqt,    &
                 0.09301981285636_adqt,   &
                 0.06273486468456_adqt,   &
                 0.02778930918493_adqt,   &
!  N=15
                 0.0009277955332396_adqt, &
                 0.005324881254380_adqt,  &
                 0.01494948831251_adqt,   &
                 0.03015294391042_adqt,   &
                 0.04982592810351_adqt,   &
                 0.07168598522528_adqt,   &
                 0.09277828135504_adqt,   &
                 0.1100543192029_adqt,    &
                 0.1209126125418_adqt,    &
                 0.1236103228459_adqt,    &
                 0.1174821237352_adqt,    &
                 0.1029369133400_adqt,    &
                 0.08124957900491_adqt,   &
                 0.05422386305719_adqt,   &
                 0.02388496257768_adqt,   &
!  N=16 
                 0.0007714118602424_adqt, &
                 0.004443946511975_adqt,  &
                 0.01254977819319_adqt,   &
                 0.02551711882156_adqt,   &
                 0.04260265070016_adqt,   &
                 0.06208231592071_adqt,   &
                 0.08160983285973_adqt,   &
                 0.09864709537668_adqt,   &
                 0.1108857393961_adqt,    &
                 0.1165977355328_adqt,    &
                 0.1148706343615_adqt,    &
                 0.1057019241225_adqt,    &
                 0.08994684678464_adqt,   &
                 0.06912603219075_adqt,   &
                 0.04508897765036_adqt,   &
                 0.01955795971710_adqt,   &

!  N=17
                 0.0005033980879584_adqt, &
                 0.002963411954252_adqt,  &
                 0.008619903084610_adqt,  &
                 0.01813659400784_adqt,   &
                 0.03141368012838_adqt,   &
                 0.04756815316756_adqt,   &
                 0.06508105325269_adqt,   &
                 0.08205441506288_adqt,   &
                 0.09651113160550_adqt,   &
                 0.1066822953425_adqt,    &
                 0.1112404525269_adqt,    &
                 0.1094493588037_adqt,    &
                 0.1012120443687_adqt,    &
                 0.08700201391745_adqt,   &
                 0.06748967807090_adqt,   &
                 0.04562281589888_adqt,   &
                 0.01844960071929_adqt,   &
!  N=18
                 0.0004535444460650_adqt, &
                 0.002646122049826_adqt,  &
                 0.007618786987172_adqt,  &
                 0.01589159962766_adqt,   &
                 0.02737618872149_adqt,   &
                 0.04139739026261_adqt,   &
                 0.05680552361525_adqt,   &
                 0.07214957225791_adqt,   &
                 0.08587880206166_adqt,   &
                 0.09654276634823_adqt,   &
                 0.1029641396646_adqt,    &
                 0.1043642048301_adqt,    &
                 0.1004272305469_adqt,    &
                 0.09129808005242_adqt,   &
                 0.07751774985588_adqt,   &
                 0.05991393459237_adqt,   &
                 0.03947666553464_adqt,   &
                 0.01727769854532_adqt    /

!  Quadratures for Azimuthal Integration in X-Y-Z Geometry

   data cosPhiXYZ /                      &
!  N=1
                 0.707106781187_adqt,    &
!  N=2
                 0.328861319306_adqt,    &
                 0.944378225429_adqt,    &
!  N=3
                 0.179705750550_adqt,    &
                 0.707106781187_adqt,    &
                 0.983720409069_adqt,    &
!  N=4
                 0.111919417802_adqt,    &
                 0.499008815360_adqt,    &
                 0.866596908714_adqt,    &
                 0.993717285710_adqt,    &
!  N=5
                 0.0760839341001_adqt,   &
                 0.359915255224_adqt,    &
                 0.707106781187_adqt,    &
                 0.932984999374_adqt,    &
                 0.997101416593_adqt,    &
!  N=6 
                 0.0549823833982_adqt,   &
                 0.268645060598_adqt,    &
                 0.565871339345_adqt,    &
                 0.824493558076_adqt,    &
                 0.963239238931_adqt,    &
                 0.998487324665_adqt,    &
!  N=7
                 0.0415504673952_adqt,   &
                 0.207031268158_adqt,    &
                 0.454812160314_adqt,    &
                 0.707106781187_adqt,    &
                 0.890587389778_adqt,    &
                 0.978334326294_adqt,    &
                 0.999136406433_adqt,    &
!  N=8
                 0.0324869788463_adqt,   &
                 0.163951256940_adqt,    &
                 0.370037938547_adqt,    &
                 0.600618666309_adqt,    &
                 0.799535626274_adqt,    &
                 0.929016643573_adqt,    &
                 0.986468441131_adqt,    &
                 0.999472158795_adqt,    &
!  N=9
                 0.0260886177621_adqt,   &
                 0.132824656907_adqt,    &
                 0.305325234799_adqt,    &
                 0.510423681971_adqt,    &
                 0.707106781187_adqt,    &
                 0.859923057537_adqt,    &
                 0.952248129951_adqt,    &
                 0.991139551485_adqt,    &
                 0.999659634087_adqt,    &

!  N=10 
                 0.0214063664984_adqt,   &
                 0.109678208079_adqt,    &
                 0.255406277071_adqt,    &
                 0.435996579025_adqt,    &
                 0.621758783706_adqt,    &
                 0.783208793927_adqt,    &
                 0.899948322449_adqt,    &
                 0.966833819037_adqt,    &
                 0.993967147683_adqt,    &
                 0.999770857484_adqt,    &
!  N=11
                 0.0178781812798_adqt,   &
                 0.0920337735274_adqt,   &
                 0.216363520647_adqt,    &
                 0.375037367368_adqt,    &
                 0.546424732468_adqt,    &
                 0.707106781187_adqt,    &
                 0.837508215928_adqt,    &
                 0.927009694166_adqt,    &
                 0.976312873485_adqt,    &
                 0.995755886013_adqt,    &
                 0.999840172545_adqt,    &
!  N=12 
                 0.0151542763381_adqt,   &
                 0.0782933178482_adqt,   &
                 0.185385682416_adqt,    &
                 0.325051390048_adqt,    &
                 0.481334101720_adqt,    &
                 0.635937834851_adqt,    &
                 0.771740286758_adqt,    &
                 0.876537211145_adqt,    &
                 0.945696353925_adqt,    &
                 0.982665837788_adqt,    &
                 0.996930366866_adqt,    &
                 0.999885167361_adqt,    &
!  N=13
                 0.0130078426034_adqt,   &
                 0.0673938148498_adqt,   &
                 0.160465382602_adqt,    &
                 0.283850626570_adqt,    &
                 0.425605349674_adqt,    &
                 0.571479597861_adqt,    &
                 0.707106781187_adqt,    &
                 0.820616274045_adqt,    &
                 0.904908882888_adqt,    &
                 0.958868511213_adqt,    &
                 0.987041468727_adqt,    &
                 0.997726452351_adqt,    &
                 0.999915394436_adqt,    &

!  N=14
                 0.0112866459910_adqt,   &
                 0.0586080282593_adqt,   &
                 0.140159297798_adqt,    &
                 0.249653542335_adqt,    &
                 0.378014529413_adqt,    &
                 0.514104408362_adqt,    &
                 0.646095281689_adqt,    &
                 0.763256763467_adqt,    &
                 0.857727612534_adqt,    &
                 0.925799662752_adqt,    &
                 0.968335225425_adqt,    &
                 0.990128966974_adqt,    &
                 0.998281072155_adqt,    &
                 0.999936303783_adqt,    &
!  N=15 
                 0.00988545479889_adqt,  &
                 0.0514258427096_adqt,   &
                 0.123417452644_adqt,    &
                 0.221052006416_adqt,    &
                 0.337331340616_adqt,    &
                 0.463490786372_adqt,    &
                 0.589865690428_adqt,    &
                 0.707106781187_adqt,    &
                 0.807501372913_adqt,    &
                 0.886101738486_adqt,    &
                 0.941385981751_adqt,    &
                 0.975262021438_adqt,    &
                 0.992354841971_adqt,    &
                 0.998676815943_adqt,    &
                 0.999951137698_adqt,    &
!  N=16
                 0.00872965710742_adqt,  &
                 0.0454813865221_adqt,   &
                 0.109465718192_adqt,    &
                 0.196944891481_adqt,    &
                 0.302448274604_adqt,    &
                 0.419021114815_adqt,    &
                 0.538777013625_adqt,    &
                 0.653724513962_adqt,    &
                 0.756732621105_adqt,    &
                 0.842448413607_adqt,    &
                 0.907976489420_adqt,    &
                 0.953165799423_adqt,    &
                 0.980414560132_adqt,    &
                 0.993990571656_adqt,    &
                 0.998965186320_adqt,    &
                 0.999961895817_adqt,    &

!  N=17
                 0.00776516751178_adqt,  &
                 0.0405069844047_adqt,   &
                 0.0977254732972_adqt,   &
                 0.176472506876_adqt,    &
                 0.272416454329_adqt,    &
                 0.379989298340_adqt,    &
                 0.492742032328_adqt,    &
                 0.603908913692_adqt,    &
                 0.707106781187_adqt,    &
                 0.797053338218_adqt,    &
                 0.870175436092_adqt,    &
                 0.924990882737_adqt,    &
                 0.962179440339_adqt,    &
                 0.984305569585_adqt,    &
                 0.995213410214_adqt,    &
                 0.999179255296_adqt,    &
                 0.999969850632_adqt,    &
!  N=18
                 0.00695206352986_adqt,  &
                 0.0363035575806_adqt,   &
                 0.0877589835173_adqt,   &
                 0.158962438822_adqt,    &
                 0.246443244015_adqt,    &
                 0.345703234725_adqt,    &
                 0.451447239310_adqt,    &
                 0.557970321081_adqt,    &
                 0.659662846613_adqt,    &
                 0.751561660011_adqt,    &
                 0.829860904485_adqt,    &
                 0.892297814701_adqt,    &
                 0.938343899378_adqt,    &
                 0.969157225366_adqt,    &
                 0.987284631220_adqt,    &
                 0.996141737311_adqt,    &
                 0.999340808587_adqt,    &
                 0.999975834114_adqt,    &
!  N=19 
                 0.00625951928174_adqt,  &
                 0.0327165178199_adqt,   &
                 0.0792219146827_adqt,   &
                 0.143872133508_adqt,    &
                 0.223857123544_adqt,    &
                 0.315511709051_adqt,    &
                 0.414465834283_adqt,    &
                 0.515911914917_adqt,    &
                 0.614970011895_adqt,    &
                 0.707106781187_adqt,    &
                 0.788550495828_adqt,    &
                 0.856641638053_adqt,    &
                 0.910064872530_adqt,    &
                 0.948921683519_adqt,    &
                 0.974621971966_adqt,    &
                 0.989596285967_adqt,    &
                 0.996857004908_adqt,    &
                 0.999464671443_adqt,    &
                 0.999980409017_adqt,    &

!  N=20
                 0.00567509348883_adqt,  &
                 0.0296820847851_adqt,   &
                 0.0719670984270_adqt,   &
                 0.130959454476_adqt,    &
                 0.204344253359_adqt,    &
                 0.289096892987_adqt,    &
                 0.381584702597_adqt,    &
                 0.477753713079_adqt,    &
                 0.573392143001_adqt,    &
                 0.664444346259_adqt,    &
                 0.747337748762_adqt,    &
                 0.819281056991_adqt,    &
                 0.878493818783_adqt,    &
                 0.924333876229_adqt,    &
                 0.957299841463_adqt,    &
                 0.978899088834_adqt,    &
                 0.991387725001_adqt,    &
                 0.997407006565_adqt,    &
                 0.999559389853_adqt,    &
                 0.999983896527_adqt,    &
!  N=21 
                 0.00488519874076_adqt,  &
                 0.0256466982317_adqt,   &
                 0.0625820489998_adqt,   &
                 0.114852883153_adqt,    &
                 0.180994199785_adqt,    &
                 0.258831435583_adqt,    &
                 0.345501744902_adqt,    &
                 0.437547785645_adqt,    &
                 0.530993475247_adqt,    &
                 0.622088626086_adqt,    &
                 0.707106781187_adqt,    &
                 0.782946831716_adqt,    &
                 0.847375907874_adqt,    &
                 0.899195159728_adqt,    &
                 0.938418107386_adqt,    &
                 0.965922506185_adqt,    &
                 0.983484163393_adqt,    &
                 0.993382512042_adqt,    &
                 0.998039822423_adqt,    &
                 0.999671069337_adqt,    &
                 0.999988067345_adqt,    &

!  N=22 
                 0.00415241892965_adqt,  &
                 0.0214943983335_adqt,   &
                 0.0551949788462_adqt,   &
                 0.104398716322_adqt,    &
                 0.167765936458_adqt,    &
                 0.243263988423_adqt,    &
                 0.328167683038_adqt,    &
                 0.419138577496_adqt,    &
                 0.512304337326_adqt,    &
                 0.603869595918_adqt,    &
                 0.690103150166_adqt,    &
                 0.723711021148_adqt,    &
                 0.797083126860_adqt,    &
                 0.858803974115_adqt,    &
                 0.907922272474_adqt,    &
                 0.944619485195_adqt,    &
                 0.969960118735_adqt,    &
                 0.985826856281_adqt,    &
                 0.994535523765_adqt,    &
                 0.998475595250_adqt,    &
                 0.999768968732_adqt,    &
                 0.999991378671_adqt     /

   data weightPhiXYZ /                   &
!  N=1
                 1.57079632679_adqt,     &
!  N=2
                 0.785398163397_adqt,    &
                 0.785398163397_adqt,    &
!  N=3
                 0.441492408805_adqt,    &
                 0.687811509185_adqt,    &
                 0.441492408805_adqt,    &
!  N=4
                 0.279049509269_adqt,    &
                 0.506348654128_adqt,    &
                 0.506348654128_adqt,    &
                 0.279049509269_adqt,    &
!  N=5 
                 0.191320724703_adqt,    &
                 0.374855576614_adqt,    &
                 0.438443724162_adqt,    &
                 0.374855576614_adqt,    &
                 0.191320724703_adqt,    &
!  N=6
                 0.138991227102_adqt,    &
                 0.284984797595_adqt,    &
                 0.361422138701_adqt,    &
                 0.361422138701_adqt,    &
                 0.284984797595_adqt,    &
                 0.138991227102_adqt,    &
!  N=7
                 0.105403262712_adqt,    &
                 0.222573419243_adqt,    &
                 0.296659466192_adqt,    &
                 0.321524030502_adqt,    &
                 0.296659466192_adqt,    &
                 0.222573419243_adqt,    &
                 0.105403262712_adqt,    &
!  N=8
                 0.0826105878919_adqt,   &
                 0.178010819682_adqt,    &
                 0.245365013149_adqt,    &
                 0.279411742675_adqt,    &
                 0.279411742675_adqt,    &
                 0.245365013149_adqt,    &
                 0.178010819682_adqt,    &
                 0.0826105878919_adqt,   &
!  N=9
                 0.0664555994585_adqt,   &
                 0.145296593486_adqt,    &
                 0.205144486413_adqt,    &
                 0.241615952092_adqt,    &
                 0.253771063895_adqt,    &
                 0.241615952092_adqt,    &
                 0.205144486413_adqt,    &
                 0.145296593486_adqt,    &
                 0.0664555994585_adqt,   &

!  N=10
                 0.0545988342772_adqt,   &
                 0.120668935327_adqt,    &
                 0.173450649536_adqt,    &
                 0.209351736725_adqt,    &
                 0.227328007532_adqt,    &
                 0.227328007532_adqt,    &
                 0.209351736725_adqt,    &
                 0.173450649536_adqt,    &
                 0.120668935327_adqt,    &
                 0.0545988342772_adqt,   &
!  N=11
                 0.0456447136792_adqt,   &
                 0.101713727266_adqt,    &
                 0.148226563396_adqt,    &
                 0.182261253182_adqt,    &
                 0.202761156967_adqt,    &
                 0.209581497814_adqt,    &
                 0.202761156967_adqt,    &
                 0.182261253182_adqt,    &
                 0.148226563396_adqt,    &
                 0.101713727266_adqt,    &
                 0.0456447136792_adqt,   &
!  N=12
                 0.0387199557399_adqt,   &
                 0.0868389882482_adqt,   &
                 0.127922016108_adqt,    &
                 0.159596288800_adqt,    &
                 0.180853870486_adqt,    &
                 0.191467044015_adqt,    &
                 0.191467044015_adqt,    &
                 0.180853870486_adqt,    &
                 0.159596288800_adqt,    &
                 0.127922016108_adqt,    &
                 0.0868389882482_adqt,   &
                 0.0387199557399_adqt,   &
!  N=13
                 0.0332559161972_adqt,   &
                 0.0749665892004_adqt,   &
                 0.111389848018_adqt,    &
                 0.140595290324_adqt,    &
                 0.161656324749_adqt,    &
                 0.174288947541_adqt,    &
                 0.178490494733_adqt,    &
                 0.174288947541_adqt,    &
                 0.161656324749_adqt,    &
                 0.140595290324_adqt,    &
                 0.111389848018_adqt,    &
                 0.0749665892004_adqt,   &
                 0.0332559161972_adqt,   &

!  N=14
                 0.0288696539886_adqt,   &
                 0.0653481659572_adqt,   &
                 0.0977815191598_adqt,   &
                 0.124592860221_adqt,    &
                 0.144949862586_adqt,    &
                 0.158538699832_adqt,    &
                 0.165317401653_adqt,    &
                 0.165317401653_adqt,    &
                 0.158538699832_adqt,    &
                 0.144949862586_adqt,    &
                 0.124592860221_adqt,    &
                 0.0977815191598_adqt,   &
                 0.0653481659572_adqt,   &
                 0.0288696539886_adqt,   &
!  N=15
                 0.0252957570834_adqt,   &
                 0.0574524794603_adqt,   &
                 0.0864651177202_adqt,   &
                 0.111038832659_adqt,    &
                 0.130436640770_adqt,    &
                 0.144335592135_adqt,    &
                 0.152659571621_adqt,    &
                 0.155428343897_adqt,    &
                 0.152659571621_adqt,    &
                 0.144335592135_adqt,    &
                 0.130436640770_adqt,    &
                 0.111038832659_adqt,    &
                 0.0864651177202_adqt,   &
                 0.0574524794603_adqt,   &
                 0.0252957570834_adqt,   &
!  N=16
                 0.0223456344014_adqt,   &
                 0.0508947121298_adqt,   &
                 0.0769654619574_adqt,   &
                 0.0994883219743_adqt,   &
                 0.117816372561_adqt,    &
                 0.131631849523_adqt,    &
                 0.140833201160_adqt,    &
                 0.145422609690_adqt,    &
                 0.145422609690_adqt,    &
                 0.140833201160_adqt,    &
                 0.131631849523_adqt,    &
                 0.117816372561_adqt,    &
                 0.0994883219743_adqt,   &
                 0.0769654619574_adqt,   &
                 0.0508947121298_adqt,   &
                 0.0223456344014_adqt,   &

!  N=17
                 0.0198823343503_adqt,   &
                 0.0453909784761_adqt,   &
                 0.0689213492840_adqt,   &
                 0.0895844718130_adqt,   &
                 0.106815271243_adqt,    &
                 0.120310163196_adqt,    &
                 0.129950917520_adqt,    &
                 0.135721804820_adqt,    &
                 0.137641745392_adqt,    &
                 0.135721804820_adqt,    &
                 0.129950917520_adqt,    &
                 0.120310163196_adqt,    &
                 0.106815271243_adqt,    &
                 0.0895844718130_adqt,   &
                 0.0689213492840_adqt,   &
                 0.0453909784761_adqt,   &
                 0.0198823343503_adqt,   &
!  N=18
                 0.0178046165668_adqt,   &
                 0.0407286438944_adqt,   &
                 0.0620554963558_adqt,   &
                 0.0810416378981_adqt,   &
                 0.0971946843126_adqt,   &
                 0.110230341949_adqt,    &
                 0.120021162996_adqt,    &
                 0.126535909639_adqt,    &
                 0.129785669785_adqt,    &
                 0.129785669785_adqt,    &
                 0.126535909639_adqt,    &
                 0.120021162996_adqt,    &
                 0.110230341949_adqt,    &
                 0.0971946843126_adqt,   &
                 0.0810416378981_adqt,   &
                 0.0620554963558_adqt,   &
                 0.0407286438944_adqt,   &
                 0.0178046165668_adqt,   &

!  N=19
                 0.0160342230649_adqt,   &
                 0.0367417542903_adqt,   &
                 0.0561476310279_adqt,   &
                 0.0736262534663_adqt,   &
                 0.0887486446168_adqt,   &
                 0.101251710769_adqt,    &
                 0.111004006334_adqt,    &
                 0.117961656788_adqt,    &
                 0.122126237364_adqt,    &
                 0.123512091351_adqt,    &
                 0.122126237364_adqt,    &
                 0.117961656788_adqt,    &
                 0.111004006334_adqt,    &
                 0.101251710769_adqt,    &
                 0.0887486446168_adqt,   &
                 0.0736262534663_adqt,   &
                 0.0561476310279_adqt,   &
                 0.0367417542903_adqt,   &
                 0.0160342230649_adqt,   &
!  N=20
                 0.0145393921491_adqt,   &
                 0.0333596736792_adqt,   &
                 0.0510954081947_adqt,   &
                 0.0672144228938_adqt,   &
                 0.0813447311863_adqt,   &
                 0.0932502149372_adqt,   &
                 0.102806017860_adqt,    &
                 0.109965085574_adqt,    &
                 0.114724634716_adqt,    &
                 0.117098582207_adqt,    &
                 0.117098582207_adqt,    &
                 0.114724634716_adqt,    &
                 0.109965085574_adqt,    &
                 0.102806017860_adqt,    &
                 0.0932502149372_adqt,   &
                 0.0813447311863_adqt,   &
                 0.0672144228938_adqt,   &
                 0.0510954081947_adqt,   &
                 0.0333596736792_adqt,   &
                 0.0145393921491_adqt,   &

!  N=21
                 0.0125272984288_adqt,   &
                 0.0289516129387_adqt,   &
                 0.0448810132722_adqt,   &
                 0.0599002608724_adqt,   &
                 0.0736350186708_adqt,   &
                 0.0856919279330_adqt,   &
                 0.0959071266252_adqt,   &
                 0.103872159085_adqt,    &
                 0.109613498442_adqt,    &
                 0.113243736140_adqt,    &
                 0.114349021980_adqt,    &
                 0.113243736140_adqt,    &
                 0.109613498442_adqt,    &
                 0.103872159085_adqt,    &
                 0.0959071266252_adqt,   &
                 0.0856919279330_adqt,   &
                 0.0736350186708_adqt,   &
                 0.0599002608724_adqt,   &
                 0.0448810132722_adqt,   &
                 0.0289516129387_adqt,   &
                 0.0125272984288_adqt,   &
!  N=22
                 0.0103702865159_adqt,   &
                 0.0252171295699_adqt,   &
                 0.0417522214778_adqt,   &
                 0.0570465053218_adqt,   &
                 0.0704500290301_adqt,   &
                 0.0838347578417_adqt,   &
                 0.0927746889748_adqt,   &
                 0.103667304767_adqt,    &
                 0.105523198981_adqt,    &
                 0.118754396046_adqt,    &
                 0.0760076448718_adqt,   &
                 0.0760076448718_adqt,   &
                 0.118754396046_adqt,    &
                 0.105523198981_adqt,    &
                 0.103667304767_adqt,    &
                 0.0927746889748_adqt,   &
                 0.0838347578417_adqt,   &
                 0.0704500290301_adqt,   &
                 0.0570465053218_adqt,   &
                 0.0417522214778_adqt,   &
                 0.0252171295699_adqt,   &
                 0.0103702865159_adqt   /



   end module QuadratureData_mod 


